﻿using System;
using System.Reflection;

namespace MT4.Test.Singleton
{
    public static class Singleton<T> where T : class
    {
        private static volatile T _instance;
        private static readonly object Lock = new object();

        public static T Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (Lock)
                    {
                        if (_instance == null)
                        {
                            ConstructorInfo constructor = null;

                            try
                            {
                                constructor = typeof (T).GetConstructor(BindingFlags.Instance | BindingFlags.NonPublic,
                                                                        null, new Type[0], null);
                            }
                            catch (Exception e)
                            {
                                throw new SingletonException(e);
                            }

                            if (constructor == null || constructor.IsAssembly) // Also exclude internal constructors.
                                throw new SingletonException(
                                    string.Format("A private or protected constructor is missing for '{0}'.",
                                                  typeof (T).Name));

                            _instance = (T) constructor.Invoke(null);
                        }
                    }
                }

                return _instance;
            }
        }
    }
}